/**
 * Javascript functions for the Save and review / Autosave plugin
 * for {@link http://b2evolution.net b2evolution}.
 *
 * @copyright (c) 2008 by Tilman Blumenbach - {@link http://ax86.net}
 * @license This file is licensed under the same license as the Save and review / Autosave plugin.
 * @package plugins
 *
 * @author Tilman Blumenbach <tilman (at) ax86 (dot) net>
 * @author ¥åßßå (yabs): jQuery
 */

/**
 * Pad a number with zeroes so that it has at least 2 digits.
 *
 * @param integer num
 * @return string The padded number
 */
function asPadNumber( num )
{
	// Convert the number to a string:
	num += '';
	if ( num.length < 2 )
	{
		num = '0' + num;
	}

	return num;
}

/**
 * Do the autosave.
 *
 * @param boolean force Force the autosave (default: false).
 */
function asAutoSave()
{
	if ( ( asAutoSave.arguments.length < 1 || ! asAutoSave.arguments[0] )
		&& ! bozo.tab_changes['item_checkchanges'] )
	{	// Do not force the autosave and there are no form changes.
		//console.log( 'Aborting autosave, no form changes.' );
		return;
	}
	
	//console.log( 'asAutoSave() called, clearing asInterval.' );
	window.clearInterval( asInterval );
	jQuery( '#autosave_state' ).empty().html( sar_lang_autosave + ' '
											+ sar_lang_saving );
	// Save the original "action" attribute and set our own action and target:
	var asCurrentAction = jQuery( '#item_checkchanges' ).attr( 'action' );
	jQuery( '#item_checkchanges' ).attr( 'action', htsrv_url + 'call_plugin.php' )
								  .attr( 'target', 'autosavefrm' ).submit();
	// Reset the form attributes: 
	jQuery( '#item_checkchanges' ).attr( 'action', asCurrentAction )
								  .removeAttr( 'target' );
}

/**
 * Handle the server reply.
 *
 * @param integer asStatus Message ID, >= 1 means error, 0 means OK.
 * @param string asMessage The message.
 */
function asServerReply( asStatus, asMessage )
{
	if( ! asStatus )
	{ // we got an OK
		date = new Date();
		jQuery( '#autosave_state' ).empty().html( sar_lang_autosave
												+ ' ' + asPadNumber( date.getHours() )
												+ ':' + asPadNumber( date.getMinutes() )
												+ ':' + asPadNumber( date.getSeconds() ) );
		// Modify bozo validator values:
		bozo.nb_changes -= bozo.tab_changes['item_checkchanges'];
		bozo.tab_changes['item_checkchanges'] = 0;
	}
	else
	{ // we got an error
		window.alert( sar_lang_error + ' ' + asMessage );
		jQuery( '#autosave_state' ).empty().html( sar_lang_autosave + ' '
												+ sar_lang_errorword );
	}
	asInterval = window.setInterval( 'asAutoSave()', sar_interval );
}

var asInterval = window.setInterval( 'asAutoSave()', sar_interval );

if ( sar_bozo_included )
{	// the bozo validator has been included by us, so we can disable the warning completely.
	window.onbeforeunload = '';
}
