<?php
/**
 * This file implements the Save and review plugin for {@link http://b2evolution.net/ b2evolution}.
 *
 * @copyright (c) 2008 by Tilman Blumenbach - {@link http://www.ax86.net/}.
 * @license GNU General Public License 2 (GPL) - http://www.opensource.org/licenses/gpl-license.php
 * @package plugins
 *
 * @author Tilman Blumenbach <tilman [AT] ax86 [DOT] net>
 * @author ¥åßßå (yabs): jQuery
 */
if( ! defined( 'EVO_MAIN_INIT' ) ) die( 'Please, do not access this page directly.' );

/**
 * Save and review Plugin
 *
 * Displays a "Save (and review)" button in simple or/and expert post
 * edit mode (see user settings). Also has a autosave function.
 *
 * @package plugins
 */
class save_and_review_plugin extends Plugin
{
    var $name               = 'Save and review / Autosave';
    var $code               = 'ax86_autosave';
    var $priority           = 50;
    var $version            = '1.1.2';
    var $author             = 'Tilman Blumenbach';
    var $help_url           = 'http://forums.b2evolution.net/viewtopic.php?p=80549';
    var $number_of_installs = 1;
    var $group              = 'editing';

    /**
     * Print a JavaScript to call the JS function asServerReply() in the
     * parent window (was the autosave OK?).
     *
     * @param int The message ID. 0 means OK, >= 1 means error.
     * @param string The message.
     */
    function _json_msg( $id, $msg )
    {
        echo '<html><head></head><body><script type="text/javascript">';
        echo 'window.parent.asServerReply( '.$id.',\''.$msg.'\' );';
        echo '</script></body></html>';
        exit();
    }

    /**
     * Delete the autosaved data from the DB.
     *
     * @param integer User ID. Defaults to the current user ID.
     * @return boolean True if data was deleted successfully.
     */
    function _delete_autosave_data( $uid = NULL )
    {
        global $DB, $current_User;

        $uid = $uid === NULL ? $current_User->ID : $uid;

        return (bool)$DB->query( 'DELETE FROM '.$DB->escape( $this->get_sql_table( 'postdata' ) )
                                .' WHERE uid = '.$DB->quote( $uid ), 'Deleting autosave data for user #'.$uid );
    }

    /**
     * Check whether autosave is enabled and there is data in the DB.
     *
     * @return boolean True if autosave is enabled and there is data in the DB.
     */
    function _check_autosave() {
        global $DB, $current_User;

        return (bool)( $this->UserSettings->get( 'sar_autosave_interval' )
                      && $DB->query( 'SELECT postdata FROM '
                                    .$DB->escape( $this->get_sql_table( 'postdata' ) )
                                    .' WHERE uid = '.$DB->quote( $current_User->ID ).' LIMIT 1',
                                     'Check for autosave data' ) );
    }

    function PluginInit( & $params )
    {
        // On uninstall, b2evo triggers SkinBeginHtmlHead() and PHP displays
        // an error because $this->UserSettings is NULL.
        if ( $params['is_installed'] && $this->UserSettings === NULL )
        {
            return false;
        }

        $this->short_desc = $this->T_( 'Displays a "Save (and review)" button '
                                      .'underneath the post form. Also has '
                                      .'a autosave feature (see long description).' );
        $this->long_desc = $this->T_( 'The autosave mechanism saves the post '
                                     .'you\'re working at every X seconds and '
                                     .'- if your post wasn\'t saved correctly - '
                                     .'pre-fills the post form the next time you '
                                     .'create/edit a post. This is a nice feature if '
                                     .'i.e. your browser crashes while you\'re '
                                     .'writing. You\'ve to finish your post '
                                     .'right after the crash or you won\'t '
                                     .'be able to create/edit other posts normally (sorry).' );
        return true;
    }

    function GetDefaultUserSettings()
    {
        return array(
            'sar_display_btn_simple' =>
                array(
                    'type' => 'checkbox',
                    'defaultvalue' => 0,
                    'label' => $this->T_( 'Display review button in simple edit mode' ),
                ),
            'sar_display_btn_expert' =>
                array(
                    'type' => 'checkbox',
                    'defaultvalue' => 1,
                    'label' => $this->T_( 'Display review button in expert edit mode' ),
                ),
            'sar_autosave_interval' =>
                array(
                    'type' => 'integer',
                    'defaultvalue' => 300,
                    'label' => $this->T_( 'Autosave interval' ),
                    'note' => $this->T_( 'Seconds. 0 to disable. 300 seconds are 5 minutes.' ),
                ),
            'sar_display_asbtn_simple' =>
                array(
                'type' => 'checkbox',
                    'defaultvalue' => 0,
                    'label' => $this->T_( 'Display autosave button in simple edit mode' ),
                ),
            'sar_display_asbtn_expert' =>
                array(
                    'type' => 'checkbox',
                    'defaultvalue' => 1,
                    'label' => $this->T_( 'Display autosave button in expert edit mode' ),
                ),
            );
    }

    /**
     * We display the autosave and "Save and review" controls here.
     *
     * @param array See the plugin docs for this hook.
     */
    function AdminDisplayEditorButton( & $params )
    {
        global $action;

        $ret = false;

        if ( $params['target_type'] == 'Item' )
        {
            if ( ( $params['edit_layout'] == 'simple'
                && $this->UserSettings->get( 'sar_display_btn_simple' ) )
                || ( $params['edit_layout'] == 'expert'
                && $this->UserSettings->get( 'sar_display_btn_expert' ) ) )
            {
                // display the review button
                echo "\n", '<input type="submit" name="review" value="';
                echo T_( 'Save (and review)' ), '" class="SaveButton" />', "\n";
                $ret = true;
            }

            if ( $this->UserSettings->get( 'sar_autosave_interval' ) )
            {
                // we're creating a new post and autosave is enabled,
                // use JS to display the autosave stuff so that users
                // without JS don't see the useless button & status area
                echo "\n", '<script type="text/javascript">', "\n\t";
                echo '<!--', "\n\t";
                if ( ( $params['edit_layout'] == 'simple'
                    && $this->UserSettings->get( 'sar_display_asbtn_simple' ) )
                    || ( $params['edit_layout'] == 'expert'
                    && $this->UserSettings->get( 'sar_display_asbtn_expert' ) ) )
                {
                    // display the autosave button
                    echo 'document.write( \'<input type="button" value="';
                    echo $this->T_( 'Autosave now' ), '" onclick="';
                    echo 'asAutoSave( true )" />\' );'."\n\t";
                }
                echo 'document.write( \'<span id="autosave_state" ';
                echo 'style="font-size:xx-small !important; margin: 0px 3px;">';
                echo $this->T_( 'Autosave:' );
                echo ' --:--:--</span><iframe width="0" height="0" frameborder="0"';
                echo ' name="autosavefrm"></iframe><input type="hidden"';
                echo ' name="plugin_ID" value="', $this->ID, '" /><input';
                echo ' type="hidden" name="method" value="savePOST" />\' );';
                echo "\n\t", '// -->', "\n", '</script>', "\n";
                $ret = true;
            }
        }
        return $ret;
    }

    /**
     * If requested, the autosave data is deleted here. Otherwise we display
     * a warning message if we've autosave data.
     */
    function SkinBeginHtmlHead()
    {
        global $action, $Messages;

        if ( is_logged_in() && $this->_check_autosave() )
        {
                // there is autosaved data in the DB
                if ( param( 'autosave_delete_data', 'boolean', 0, false, false, true, true ) )
                {
                    // delete the data
                    if( $this->_delete_autosave_data() )
                    {
                        $Messages->add( $this->T_( 'Autosave data has been deleted.' ), 'success' );
                    } else {
                        $Messages->add( $this->T_( 'Could not delete autosave data.' ), 'error' );
                    }
                } else {
                    $msg = $this->T_( 'Welcome back! Please note that you have an autosaved post, '
                                     .'so hurry and finish your work (you won\'t be able to '
                                     .'create or edit other posts normally before your autosaved '
                                     .'post is finally saved). [<a %s>Delete autosave data</a>]' );
                    $set = array( 'autosave_delete_data=1' );

                    // regenerate_url() doesn't always include the action param (-> not memorized??)...
                    if ( $action == 'new' || $action == 'edit' )
                    {
                        $set[] = 'action='.$action;
                    }
                    $Messages->add( sprintf( $msg, 'href="'.regenerate_url( '', $set ).'"' ), 'note' );
                }
                return true;
        }
        return false;
    }

    /**
     * In this method, SkinBeginHtmlHead is called to display the warning
     * message/delete the autosave data also in the backoffice.
     * We output the Javascript code and restore $_POST, too.
     * A few special calls are also made here, to fill the edit/create
     * form properly.
     */
    function AdminEndHtmlHead()
    {
        global $ctrl, $action, $admin_url, $edited_Item, $Blog, $blog;
        global $DB, $allow_cross_posting, $Messages, $UserSettings;

        // display the warning message also in the backoffice
        $ret = $this->SkinBeginHtmlHead();

        if ( $ctrl == 'items' )
        {
            if ( ( $action == 'new' || $action == 'edit' ) && $this->_check_autosave() )
            {
                $row =& $DB->get_row();
                // we're creating/editing a post and there is autosaved data
                if ( ( $data = @unserialize( $row->postdata ) ) === false
                    || !is_array( $data ) )
                {
                    $this->debug_log( 'Data is corrupt, deleting.' );
                    $this->_delete_autosave_data();
                    $Messages->add( $this->T_( 'The autosave data was corrupted and has been deleted.' ), 'error' );
                } else {
                    $this->debug_log( 'Overriding/setting $_POST.' );
                    // TODO: not sure if array_merge is really needed
                    $_POST = array_merge( $_POST, $data );
                    //$this->debug_log('$_POST = '.var_export($_POST, true));

                    // override all the basic params
                    // on all param() calls we set $override = true
                    // this code is from the items controller (action new/new_switchtab)
                    $edited_Item->status = param( 'post_status', 'string', NULL, false, true ); // force!
                    $edited_Item->status = $Blog->get_allowed_item_status( $edited_Item->status );

                    param( 'post_extracats', 'array', array(), false, true );
                    param( 'edit_date', 'integer', 0, false, true );
                    param( 'post_comment_status', 'string', 'open', false, true );
                    param( 'item_tags', 'string', '', false, true );
                    param( 'trackback_url', 'string', '', false, true );

                    $edited_Item->main_cat_ID = param( 'post_category', 'integer', $Blog->get_default_cat_ID(), false, true );
                    if( $edited_Item->main_cat_ID && $allow_cross_posting < 3 && get_catblog( $edited_Item->main_cat_ID ) != $blog )
                    { // the main cat is not in the list of categories; this happens, if the user switches blogs during editing:
                      // (in our case it happens when the user autosaved the post in another blog)
                        $edited_Item->main_cat_ID = $Blog->get_default_cat_ID();
                    }

                    // finally load some params from the request
                    $edited_Item->load_from_Request( true );
                }
                $ret = true;
            }

            if ( ( substr( $action, 0, 3 ) == 'new' || substr( $action, 0, 4 ) == 'edit' )
                && ( $interval = $this->UserSettings->get( 'sar_autosave_interval' ) ) )
            {   // display the JS
                $include_bozo = ! $UserSettings->get( 'control_form_abortions' );
            
                echo "\n\t", '<script type="text/javascript">', "\n\t\t", '<!--', "\n\t\t";
                echo '// Variables used by the ', $this->name, ' plugin:', "\n\t\t";
                echo 'var sar_interval       = ', $interval, '000;', "\n\t\t";
                echo 'var sar_lang_autosave  = "', $this->T_( 'Autosave:' ), '";', "\n\t\t";
                echo 'var sar_lang_saving    = "', $this->T_( 'Saving...' ), '";', "\n\t\t";
                echo 'var sar_lang_error     = "';
                echo /* TRANS: check the newline char in _global.php (has to be \\n [two backslashes, one n]) */ $this->T_( 'Could not autosave your post!\nError was:' ), '";', "\n\t\t";
                echo 'var sar_lang_errorword = "', T_( 'Error' ), '";', "\n\t\t";
                echo 'var sar_bozo_included  = ', $include_bozo ? 'true' : 'false', ';', "\n\t\t";
                echo '// -->', "\n\t", '</script>', "\n";

                if ( $include_bozo )
                {
                    require_js( 'bozo_validator.js' );
                }
                require_js( $this->get_plugin_url( true ).'_sar.funcs.js' );
                
                $ret = true;
            }
        }
        return $ret;
    }

    /**
     * Here we override the redirect after the form has been submitted
     * (and the review button was pressed) and delete the autosave data.
     */
    function AfterItemInsert( & $params )
    {
        global $admin_url, $ctrl, $action, $redirect_to;
        static $called = false;

        // TODO: is_logged_in() needed?
        if ( $called || ! is_logged_in() )
        {
            // already called, don't do the work twice
            $this->debug_log( 'AfterItemUpdate(): returning (already called/user not logged in)' );
            return;
        }
        $called = true;

        if ( $ctrl == 'items' && ( $action == 'update' || $action == 'create' ) )
        {
            if ( isset( $_POST['review'] ) )
            {
                // override the redirect
                $this->debug_log( 'Overriding redirect (post ID: '.$params['Item']->ID.')' );
                set_param( 'redirect_to',
                        url_add_param( $admin_url, 'ctrl=items&action=edit&p='
                                                  .$params['Item']->ID
                                                  .( ! is_null( $redirect_to ) ?
                                                            ( '&redirect_to='.urlencode( $redirect_to ) )
                                                            : '' ), '&' ) );
            }

            if ( $this->_check_autosave() )
            {
                // user has saved the post and there's data in the DB
                // autosave data isn't needed anymore, delete it.
                $this->debug_log( 'Deleting autosave data.' );
                $this->_delete_autosave_data();
            }
        }
    }

    /**
     * This hook calls AfterItemInsert to override the redirect/delete
     * the autosave data when we're creating a post.
     */
    function AfterItemUpdate( & $params )
    {
        $this->AfterItemInsert( $params );
    }

    function AfterUserDelete( & $params )
    {
        $this->_delete_autosave_data( $params['User']->ID );
    }

    function GetHtsrvMethods()
    {
        return array(
            'savePOST',
        );
    }

    /**
     * Save $_POST params for current user in our DB table (update or insert).
     * Needed by the Javascript.
     */
    function htsrv_savePOST()
    {
        global $current_User, $DB;

        if ( !is_logged_in() )
        {
            $this->_json_msg( 1, 'Not logged in!' ); // EXITS
        }

        // unset some not needed (hidden) vars
        // feel free to modify as needed...
        unset( $_POST['plugin_ID'], $_POST['method'], $_POST['params'],
               $_POST['ctrl'], $_POST['action'] , $_POST['mode'],
               $_POST['redirect_to'], $_POST['preview'], $_POST['more'],
               $_POST['preview_userid'], $_POST['blog'], $_POST['post_ID'] );

        if ( ! count( $_POST ) )
        {
            $this->_json_msg( 2, 'No POST data.' );
        }

        $DB->show_errors = false;

        if ( ( $num = $DB->query( 'SELECT uid FROM '.$DB->escape( $this->get_sql_table( 'postdata' ) )
                              .' WHERE uid = '.$DB->quote( $current_User->ID ).' LIMIT 1' ) ) === false )
        {
            $this->_json_msg( 3, 'DB error (select): '.$DB->last_error );
        }

        $sql = '';
        if ( $num )
        {
            // record already exists, update
            $sql = 'UPDATE '.$DB->escape( $this->get_sql_table( 'postdata' ) )
                  .' SET postdata = '.$DB->quote( serialize( $_POST ) )
                  .' WHERE uid = '.$DB->quote( $current_User->ID );
        } else {
            // no record, create a new one
            $sql = 'INSERT INTO '.$DB->escape( $this->get_sql_table( 'postdata' ) )
                  .' (uid, postdata) VALUES ('.$DB->quote( $current_User->ID )
                  .', '.$DB->quote( serialize( $_POST ) ).')';
        }

        if ( $DB->query( $sql ) === false )
        {
            $this->_json_msg( 4, 'DB error (insert/update): '.$DB->last_error );
        }

        $this->_json_msg( 0, 'OK' );
    }

    function GetDbLayout()
    {
        global $DB;

        return array(
            'CREATE TABLE '.$DB->escape( $this->get_sql_table( 'postdata' ) )
           .' (uid INT UNSIGNED NOT NULL PRIMARY KEY, postdata LONGTEXT NOT NULL)',
        );
    }
}
?>
